package org.processmining.streamconformance.local.plugin;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.streamconformance.local.model.LocalConformanceTracker;
import org.processmining.streamconformance.local.model.LocalModelStructure;
import org.processmining.streamconformance.local.model.OnlineConformanceScore;
import org.processmining.streamconformance.utils.XLogHelper;

public class LocalOfflineConformancePlugin {

	@Plugin(
		name = "Offline Conformance - Local behavior",
		returnLabels = { "A configuration for local offline conformance checker" },
		returnTypes = { XLog.class },
		parameterLabels = {
			"Model", "Marking", "Event log"
		},
		categories = PluginCategory.Analytics,
		help = "This plugin computes the conformance of a given model with respect to an event streams.",
		userAccessible = true)
	@UITopiaVariant(
		author = "A. Burattin",
		email = "",
		affiliation = "DTU")
	public static XLog plugin(UIPluginContext context, Petrinet net, Marking initMarking, XLog log) throws Exception {
		
		LocalModelStructure lms =  new LocalModelStructure(context, net, initMarking);
		LocalConformanceTracker lct = new LocalConformanceTracker(lms, log.size());
		XLogInfo info = XLogInfoFactory.createLogInfo(log);
		
		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(info.getNumberOfEvents());
		
		XLog enriched = XLogHelper.generateNewXLog(XLogHelper.getName(log));
		for (XTrace origTrace : log) {
			String traceName = XLogHelper.getName(origTrace);
			XTrace newTrace = XLogHelper.createTrace(traceName);
			for (XEvent origEvent : origTrace) {
				String eventName = XLogHelper.getName(origEvent);
				OnlineConformanceScore ocs = lct.replayEvent(traceName, eventName);
				XEvent newEvent = XLogHelper.insertEvent(newTrace, eventName);
				XLogHelper.decorateElement(newEvent, "conformance", ocs.getConformance());
				XLogHelper.decorateElement(newEvent, "confidence", ocs.getConfidence());
				XLogHelper.decorateElement(newEvent, "completeness", ocs.getCompleteness());
				context.getProgress().inc();
			}
			enriched.add(newTrace);
		}
		
		return enriched;
	}
}
