package org.processmining.streamconformance.local.tests;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.Iterator;

import org.apache.commons.io.FileUtils;
import org.processmining.contexts.cli.CLIContext;
import org.processmining.contexts.cli.CLIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.boot.Boot;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Bootable;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.CommandLineArgumentList;
import org.processmining.framework.util.Pair;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.connections.transitionsystem.TransitionSystemConnection;
import org.processmining.models.graphbased.directed.DirectedGraphElementWeights;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.transitionsystem.AcceptStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.CoverabilityGraph;
import org.processmining.models.graphbased.directed.transitionsystem.StartStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystemFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.petrinet.behavioralanalysis.CGGenerator;
import org.processmining.plugins.pnml.importing.PnmlImportNet;
import org.processmining.plugins.tsml.Tsml;
import org.processmining.plugins.tsml.importing.TsmlImportUtils;
import org.processmining.streamconformance.local.model.LocalConformanceTracker;
import org.processmining.streamconformance.local.model.LocalModelStructure;
import org.processmining.streamconformance.local.model.OnlineConformanceScore;
import org.processmining.streamconformance.local.plugin.LocalOnlineConformanceConfiguration;
import org.processmining.streamconformance.utils.PetrinetHelper;

public class CorrelationTest {

	static String basePath = "C:\\Users\\andbur\\Desktop\\tests\\";
	
	
	public static void saveTSML(PluginContext context, TransitionSystem ts, File file) throws IOException {
		StartStateSet starts = new StartStateSet();
		AcceptStateSet accepts = new AcceptStateSet();
		DirectedGraphElementWeights weights = new DirectedGraphElementWeights();

		ConnectionManager cm = context.getConnectionManager();
		try {
			TransitionSystemConnection tsc = cm.getFirstConnection(TransitionSystemConnection.class, context, ts);
			if (tsc.hasWeights()) {
				weights = tsc.getObjectWithRole(TransitionSystemConnection.WEIGHTS);
			}
			starts = tsc.getObjectWithRole(TransitionSystemConnection.STARTIDS);
			accepts = tsc.getObjectWithRole(TransitionSystemConnection.ACCEPTIDS);

		} catch (ConnectionCannotBeObtained e) {
			/**
			 * No connected transition weights found, no problem.
			 */
		}
		GraphLayoutConnection layout;
		try {
			layout = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, ts);
		} catch (ConnectionCannotBeObtained e) {
			layout = new GraphLayoutConnection(ts);
		}

		Tsml tsml = new Tsml().marshall(ts, starts, accepts, weights, layout);
		String text = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n" + tsml.exportElement(tsml);

		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
		bw.write(text);
		bw.close();
		System.out.println("saved");
	}
	
	public static CoverabilityGraph loadTSML(PluginContext context, File file) throws Exception {
		TsmlImportUtils utils = new TsmlImportUtils();
		Tsml tsml = utils.importTsmlFromStream(context, new FileInputStream(file), file.getAbsolutePath(), -1);
		CoverabilityGraph ts = TransitionSystemFactory.newCoverabilityGraph(tsml.getLabel() + " (imported from "
				+ file + ")");
		StartStateSet starts = new StartStateSet();
		AcceptStateSet accepts = new AcceptStateSet();
		DirectedGraphElementWeights weights = new DirectedGraphElementWeights();
		GraphLayoutConnection layout = new GraphLayoutConnection(ts);
		tsml.unmarshall(ts, starts, accepts, weights, layout);
		context.addConnection(new TransitionSystemConnection(ts, weights, starts, accepts));
		context.addConnection(layout);
		return ts;
	}
	
	public static Object[] getCachedStructures(PluginContext context, Petrinet net, Marking initMarking, String filename) throws Exception {
		
		String cachePath = basePath + "cache\\" + filename + "-";
		
		// build coverability graphs
		CoverabilityGraph coverabilityGraph = null;
		File cg = new File(cachePath + "cg");
		if (cg.exists()) {
			coverabilityGraph = loadTSML(context, cg);
		} else {
			coverabilityGraph = context.tryToFindOrConstructFirstNamedObject(
					CoverabilityGraph.class,
					CGGenerator.class.getAnnotation(Plugin.class).name(),
					null,
					null,
					net,
					initMarking);
			saveTSML(context, coverabilityGraph, cg);
		}
		
		// build coverability graph of unfolded net
		CoverabilityGraph coverabilityGraphUnfolded = null;
		File cgun = new File(cachePath + "cg-un");
		if (cgun.exists()) {
			coverabilityGraphUnfolded = loadTSML(context, cgun);
		} else {
			Pair<Petrinet, Marking> unfoldedTotal = PetrinetHelper.unfold(context, net);
			coverabilityGraphUnfolded = context.tryToFindOrConstructFirstNamedObject(
					CoverabilityGraph.class,
					CGGenerator.class.getAnnotation(Plugin.class).name(),
					null,
					null,
					unfoldedTotal.getFirst(),
					unfoldedTotal.getSecond());
			saveTSML(context, coverabilityGraphUnfolded, cgun);
		}
		
		// build coverability graph of dual net
		CoverabilityGraph coverabilityGraphDualUnfolded = null;
		File cgrevun = new File(cachePath + "cg-rev-un");
		if (cgrevun.exists()) {
			coverabilityGraphDualUnfolded = loadTSML(context, cgrevun);
		} else {
			Pair<Petrinet, Marking> dualNet = PetrinetHelper.computeDual(context, net);
			Pair<Petrinet, Marking> unfoldedDualNet = PetrinetHelper.unfold(context, dualNet.getFirst());
			coverabilityGraphDualUnfolded = context.tryToFindOrConstructFirstNamedObject(
					CoverabilityGraph.class,
					CGGenerator.class.getAnnotation(Plugin.class).name(),
					null,
					null,
					unfoldedDualNet.getFirst(),
					unfoldedDualNet.getSecond());
			saveTSML(context, coverabilityGraphDualUnfolded, cgrevun);
		}
		
		return new Object[] {coverabilityGraph, coverabilityGraphUnfolded, coverabilityGraphDualUnfolded};
	}
	
	
	@Plugin(
			name = "local-test",
			parameterLabels = { "Model", "Marking", "filename" },
			returnLabels = { "" },
			returnTypes = { LocalOnlineConformanceConfiguration.class },
			userAccessible = true)
	@UITopiaVariant(
			author = "A. Burattin",
			affiliation = "",
			email = "")
	public static LocalOnlineConformanceConfiguration plugin(PluginContext context, Petrinet net, Marking initMarking, String filename) throws Exception {
		
		Object[] res = getCachedStructures(context, net, initMarking, filename);
		CoverabilityGraph coverabilityGraph = (CoverabilityGraph) res[0];
		CoverabilityGraph coverabilityGraphUnfolded = (CoverabilityGraph) res[1];
		CoverabilityGraph coverabilityGraphDualUnfolded = (CoverabilityGraph) res[2];
		
		// create the local structure
		LocalOnlineConformanceConfiguration locc = new LocalOnlineConformanceConfiguration();
		locc.setLocalModelStructure(new LocalModelStructure(coverabilityGraph, coverabilityGraphUnfolded, coverabilityGraphDualUnfolded));
		locc.setNoMaxParallelInstances(100); // TODO: hardcoded for the moment
		return locc;
	}
	
	public static LocalConformanceTracker getLCT(CLIPluginContext context, File modelFilename) throws ConnectionCannotBeObtained {
		Petrinet net = context.tryToFindOrConstructFirstNamedObject(
				Petrinet.class,
				PnmlImportNet.class.getAnnotation(Plugin.class).name(),
				null,
				null,
				modelFilename.getAbsolutePath());
		InitialMarkingConnection imc = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, net);
		
		LocalOnlineConformanceConfiguration locc = context.tryToFindOrConstructFirstNamedObject(
				LocalOnlineConformanceConfiguration.class, "local-test", null, null,
				imc.getObjectWithRole("Net"), imc.getObjectWithRole("Marking"), modelFilename.getName());
		
		LocalModelStructure lms = locc.getLocalModelStructure();
		LocalConformanceTracker lct = new LocalConformanceTracker(lms, 100);
		
//		lms.printNicely(System.out);
		return lct;
	}
	
	public static void processLog(LocalConformanceTracker lct, String logFile, String targetFile) throws IOException {
		if (new File(targetFile).exists()) {
			return;
		}
		
		String separator = "\t";
		PrintWriter writer = new PrintWriter(targetFile);
		writer.println(
				"T:concept:name" + separator +
				"E:concept:name" + separator +
				"id" + separator +
				"conformance" + separator +
				"completeness" + separator +
				"confidence");
		
		BufferedReader br = new BufferedReader(new FileReader(logFile));
		br.readLine();
		String line;
		while ((line = br.readLine()) != null) {
			String[] parts = line.split("\t");
			String eventId = parts[2];
			String caseId = parts[0];
			String activityName = parts[1];
			OnlineConformanceScore ocs = lct.replayEvent(caseId, activityName);
			writer.println(
					caseId + separator +
					activityName + separator +
					eventId + separator +
					ocs.getConformance() + separator +
					ocs.getCompleteness() + separator +
					ocs.getConfidence());
//			System.out.println(caseId + separator +
//					activityName + separator +
//					eventId + separator +
//					ocs.getConformance() + separator +
//					ocs.getCompleteness() + separator +
//					ocs.getConfidence());
		}
		
		br.close();
		writer.close();
	}
	
	@Plugin(name = "CLI", parameterLabels = {}, returnLabels = {}, returnTypes = {}, userAccessible = false)
	@Bootable
	public Object main(CommandLineArgumentList commandlineArguments) throws ConnectionCannotBeObtained, IOException {
		CLIContext globalContext = new CLIContext();
		CLIPluginContext context = new CLIPluginContext(globalContext, "c");
		System.out.flush();
		System.err.flush();
		
		String modelsPath = basePath + "models\\";
		String logsPath = basePath + "logs\\";
		String resultsPath = basePath + "results\\";
		
		String[] traceNoiseProbabilities = new String[] {"0.1", "0.2", "0.3", "0.4", "0.5"};
		String[] eventNoiseProbabilities = new String[] {"0.1", "0.2", "0.3", "0.4", "0.5"};
		
		Iterator<File> modelsIterator = FileUtils.iterateFiles(new File(modelsPath), new String[] {"pnml" }, false);
		while (modelsIterator.hasNext()) {
			File model = modelsIterator.next();
			System.out.println(model.getAbsolutePath());
			System.out.flush();
			LocalConformanceTracker lct = getLCT(context, model);
			for (String traceNoise : traceNoiseProbabilities) {
				for (String eventNoise : eventNoiseProbabilities) {
					String logFileName = "log_" + model.getName() + "_noise_trace_" + traceNoise + "_noise_event_" + eventNoise + ".csv";
					System.out.println(logFileName);
					processLog(lct, logsPath + logFileName, resultsPath + logFileName);
				}
			}
		}
		System.out.println("Done");
		System.out.flush();
		return null;
	}
	
	
	public static void main(String[] args) throws Exception {
		Boot.boot(CorrelationTest.class, CLIPluginContext.class, args);
	}
}
