package org.processmining.streamconformance.local.tests;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.processmining.contexts.cli.CLIContext;
import org.processmining.contexts.cli.CLIPluginContext;
import org.processmining.framework.boot.Boot;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Bootable;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.CommandLineArgumentList;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.pnml.importing.PnmlImportNet;
import org.processmining.streamconformance.utils.PetrinetHelper;

public class ErrorsFinder {

	public static int hasErrors(PluginContext context, File modelFilename) throws ConnectionCannotBeObtained {
		Petrinet net = context.tryToFindOrConstructFirstNamedObject(
				Petrinet.class,
				PnmlImportNet.class.getAnnotation(Plugin.class).name(),
				null,
				null,
				modelFilename.getAbsolutePath());

		if (!PetrinetHelper.isWorkflowNet(net)) {
			return -2;
		}
		
		Set<String> names = new HashSet<String>();
		for (Transition t : net.getTransitions()) {
			String name = t.getLabel();
			if (!name.startsWith("tau ")) {
				if (names.contains(name)) {
					System.out.println("  >> `" + name + "' is duplicate");
					return -1;
				}
				names.add(name);
			}
		}
		return 0;
	}
	
	@Plugin(name = "CLI", parameterLabels = {}, returnLabels = {}, returnTypes = {}, userAccessible = false)
	@Bootable
	public static void runner(CommandLineArgumentList commandlineArguments) throws ConnectionCannotBeObtained {
		CLIContext globalContext = new CLIContext();
		CLIPluginContext context = new CLIPluginContext(globalContext, "c");
		
		String path = "C:\\Users\\andbur\\Desktop\\tests\\models";
		Iterator<File> modelsIterator = FileUtils.iterateFiles(new File(path), new String[] {"pnml" }, false);
		while (modelsIterator.hasNext()) {
			File model = modelsIterator.next();
			System.out.println(model.getAbsolutePath());
			int err = hasErrors(context, model);
			if (err == -1) {
				System.out.println(" +++ has duplicates");
			} else if (err == -2) {
				System.out.println(" +++ is not wf net");
			}
		}
		System.out.println("Done");
	}
	
	public static void main(String[] args) throws Exception {
		Boot.boot(ErrorsFinder.class, CLIPluginContext.class, args);
	}
}

