package org.processmining.streamconformance.local.tests;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.cli.CLIContext;
import org.processmining.contexts.cli.CLIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.boot.Boot;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Bootable;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.CommandLineArgumentList;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.operationalsupport.xml.OSXMLConverter;
import org.processmining.plugins.pnml.importing.PnmlImportNet;
import org.processmining.streamconformance.local.model.LocalConformanceTracker;
import org.processmining.streamconformance.local.model.LocalModelStructure;
import org.processmining.streamconformance.local.plugin.LocalOnlineConformanceConfiguration;

public class StressTest {
	@Plugin(
			name = "local-test",
			parameterLabels = { "Model", "Marking" },
			returnLabels = { "" },
			returnTypes = { LocalOnlineConformanceConfiguration.class },
			userAccessible = true)
	@UITopiaVariant(
			author = "A. Burattin",
			affiliation = "",
			email = "")
	public static LocalOnlineConformanceConfiguration plugin(PluginContext context, Petrinet net, Marking initMarking) throws Exception {

		// create the local structure
		LocalOnlineConformanceConfiguration locc = new LocalOnlineConformanceConfiguration();
		locc.setLocalModelStructure(new LocalModelStructure(context, net, initMarking));
		locc.setNoMaxParallelInstances(100); // TODO: hardcoded for the moment
		
		return locc;
	}
	
	@Plugin(name = "CLI", parameterLabels = {}, returnLabels = {}, returnTypes = {}, userAccessible = false)
	@Bootable
	public Object main(CommandLineArgumentList commandlineArguments) throws ConnectionCannotBeObtained, FileNotFoundException, IOException {
		CLIContext globalContext = new CLIContext();
		CLIPluginContext context = new CLIPluginContext(globalContext, "c");
		
		System.out.println("Importing model....");
		Petrinet netUnfolded = context.tryToFindOrConstructFirstNamedObject(
				Petrinet.class,
				PnmlImportNet.class.getAnnotation(Plugin.class).name(),
				null,
				null,
				System.getProperty("user.dir") + "\\tests\\testfiles\\stress-test-2\\model.pnml");
		InitialMarkingConnection imc = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, netUnfolded);
		
		LocalOnlineConformanceConfiguration locc = context.tryToFindOrConstructFirstNamedObject(
				LocalOnlineConformanceConfiguration.class, "local-test", null, null,
				imc.getObjectWithRole("Net"), imc.getObjectWithRole("Marking"));
		
		LocalModelStructure lms = locc.getLocalModelStructure();
		LocalConformanceTracker lct = new LocalConformanceTracker(lms, 100);
		
		double totalEvents = 0;
		double totalTime = 0;
		double movingAverage = 0;
		try (BufferedReader br = new BufferedReader(new FileReader(
				System.getProperty("user.dir") + "\\tests\\testfiles\\stress-test-2\\stream"))) {
			OSXMLConverter converter = new OSXMLConverter();
			String line;
			XTrace t;
			while ((line = br.readLine()) != null) {
				t = (XTrace) converter.fromXML(line);
				String caseId = XConceptExtension.instance().extractName(t);
				String newEventName = XConceptExtension.instance().extractName(t.get(0));
				double start = System.currentTimeMillis();
				lct.replayEvent(caseId, newEventName);
				double end = System.currentTimeMillis();
				totalTime += (end - start);
				movingAverage += (end - start);
				totalEvents++;
				if (totalEvents % 10000 == 0) {
					Runtime rt = Runtime.getRuntime();
					long usedMB = (rt.totalMemory() - rt.freeMemory()) / 1024 / 1024;
					System.out.println("Processing... (" + totalEvents + " events, " + totalTime + "ms time, " + (totalTime/totalEvents) + "ms/events, " + (movingAverage/10000) + " ms/event moving average, " + usedMB + "MB)");
					movingAverage = 0;
				}
			}
		}
		
		System.out.println("Total events processed: " + totalEvents);
		System.out.println("Total time used: " + totalTime + "ms");
		System.out.println("Average time used per event: " + (totalTime/totalEvents) + "ms");
		
		return null;
	}
	
	public static void main(String[] args) throws Exception {
		Boot.boot(StressTest.class, CLIPluginContext.class, args);
	}
}
