package org.processmining.streamconformance.local.tests;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.processmining.contexts.cli.CLIContext;
import org.processmining.contexts.cli.CLIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.boot.Boot;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Bootable;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.CommandLineArgumentList;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.pnml.importing.PnmlImportNet;
import org.processmining.streamconformance.local.model.LocalConformanceTracker;
import org.processmining.streamconformance.local.model.LocalModelStructure;
import org.processmining.streamconformance.local.model.OnlineConformanceScore;
import org.processmining.streamconformance.local.plugin.LocalOnlineConformanceConfiguration;

public class Tester {
	@Plugin(
			name = "local-test",
			parameterLabels = { "Model", "Marking" },
			returnLabels = { "" },
			returnTypes = { LocalOnlineConformanceConfiguration.class },
			userAccessible = true)
	@UITopiaVariant(
			author = "A. Burattin",
			affiliation = "",
			email = "")
	public static LocalOnlineConformanceConfiguration plugin(PluginContext context, Petrinet net, Marking initMarking) throws Exception {

		// create the local structure
		LocalOnlineConformanceConfiguration locc = new LocalOnlineConformanceConfiguration();
		locc.setLocalModelStructure(new LocalModelStructure(context, net, initMarking));
		locc.setNoMaxParallelInstances(100); // TODO: hardcoded for the moment
		
		return locc;
	}
	
	@Plugin(name = "CLI", parameterLabels = {}, returnLabels = {}, returnTypes = {}, userAccessible = false)
	@Bootable
	public Object main(CommandLineArgumentList commandlineArguments) throws ConnectionCannotBeObtained {
		System.out.println("1");
		CLIContext globalContext = new CLIContext();
		System.out.println("2");
		CLIPluginContext context = new CLIPluginContext(globalContext, "c");
		System.out.println("3");
		
		System.out.println("Importing model....");
		Petrinet netUnfolded = context.tryToFindOrConstructFirstNamedObject(
				Petrinet.class,
				PnmlImportNet.class.getAnnotation(Plugin.class).name(),
				null,
				null,
				"C:\\Users\\andbur\\Desktop\\online-demo-1.pnml");
		InitialMarkingConnection imc = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, netUnfolded);
		
		LocalOnlineConformanceConfiguration locc = context.tryToFindOrConstructFirstNamedObject(
				LocalOnlineConformanceConfiguration.class, "local-test", null, null,
				imc.getObjectWithRole("Net"), imc.getObjectWithRole("Marking"));
		
		LocalModelStructure lms = locc.getLocalModelStructure();
		LocalConformanceTracker lct = new LocalConformanceTracker(lms, 100);
		
		lms.printNicely(System.out);
		
		List<List<String>> logExample1 = new LinkedList<List<String>>();
		logExample1.add(Arrays.asList("B;C;D".split(";")));
//		logExample1.add(Arrays.asList(new String[] { "A", "A1", "B", "C" }));
//		logExample1.add(Arrays.asList(new String[] { "A", "A1", "A2", "A1", "A2", "A1" }));
//		logExample1.add(Arrays.asList(new String[] { "A", "B", "C" }));
//		logExample1.add(Arrays.asList(new String[] { "B", "C", "E", "F" }));
		
		int i = 0;
		for (List<String> trace : logExample1) {
			String caseId = "case " + i;
			System.out.println("Processing: " + trace);
			for (String event : trace) {
				OnlineConformanceScore c = lct.replayEvent(caseId, event);
				System.out.println(" >> " + event + "; " + c);
			}
			i++;
		}
		
		return null;
	}
	
	public static void main(String[] args) throws Exception {
		Boot.boot(Tester.class, CLIPluginContext.class, args);
	}
}
