package org.processmining.streamconformance.regions.gui.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.util.Pair;
import org.processmining.models.connections.transitionsystem.TransitionSystemConnection;
import org.processmining.models.graphbased.directed.DirectedGraphElementWeights;
import org.processmining.models.graphbased.directed.transitionsystem.AcceptStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.CoverabilityGraph;
import org.processmining.models.graphbased.directed.transitionsystem.StartStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.State;
import org.processmining.models.graphbased.directed.transitionsystem.Transition;
import org.processmining.plugins.utils.ProvidedObjectHelper;
import org.processmining.streamconformance.regions.model.ConformanceTracker;
import org.processmining.streamconformance.regions.utils.GUICustomUtils;
import org.processmining.streamconformance.regions.utils.Triple;
import org.processmining.streamconformance.regions.utils.UIColors;
import org.processmining.streamconformance.regions.widgets.renderers.FrequentErrorListEntryRenderer;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class MostFrequentErrors extends JPanel {

	private static final long serialVersionUID = -4397031803614016435L;
	public static String DATA_OBJ_PREFIX = "[dummy-for-conformance-visualization]";

	protected UIPluginContext context;
	private CoverabilityGraph originalModel;
	private ConformanceTracker tracker;

	private DefaultListModel<Pair<Integer, Triple<State, String, State>>> dlm;
	private JList<Pair<Integer, Triple<State, String, State>>> list;
	private JPanel coverabilityGraphvisualizer;
	private JLabel nothingToDisplay;

	public MostFrequentErrors(UIPluginContext context, CoverabilityGraph originalModel, ConformanceTracker tracker) {
		this.context = context;
		this.originalModel = originalModel;
		this.tracker = tracker;

		this.dlm = new DefaultListModel<Pair<Integer, Triple<State, String, State>>>();
		this.list = new JList<Pair<Integer, Triple<State, String, State>>>(dlm);
		this.list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.list.setBackground(UIColors.lightLightGray);
		this.list.setCellRenderer(new FrequentErrorListEntryRenderer());

		this.nothingToDisplay = new JLabel("Select an error to have a graphical representation...", JLabel.CENTER);
		this.nothingToDisplay.setForeground(UIColors.darkGray);

		initComponents();
	}

	private void initComponents() {
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setOpaque(false);

		// errors list
		JScrollPane scrollPane = new JScrollPane(list);
		scrollPane.setBackground(UIColors.lightLightGray);
		scrollPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		GUICustomUtils.customizeScrollBard(scrollPane);
		list.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						// perform the operation only if something has actually been selected
						if (!list.isSelectionEmpty()) {
							// tmp selection
							coverabilityGraphvisualizer.removeAll();
							nothingToDisplay.setText("Updating model...");
							coverabilityGraphvisualizer.add(nothingToDisplay, BorderLayout.CENTER);

							// components extraction
							Pair<Integer, Triple<State, String, State>> error = list.getSelectedValue();
							CoverabilityGraph cg = cloneModel(originalModel);
							DirectedGraphElementWeights weights = new DirectedGraphElementWeights();
							StartStateSet ss = new StartStateSet();
							AcceptStateSet as = new AcceptStateSet();

							// Pair<Integer, Triple<State, String, State>>

							// add the actual error
							String id = error.getSecond().getSecond();
							cg.addTransition(error.getSecond().getFirst().getIdentifier(), error.getSecond().getThird().getIdentifier(), id);

							ArrayList<Object> list = new ArrayList<Object>(3);
							list.add(error.getSecond().getFirst().getIdentifier());
							list.add(error.getSecond().getThird().getIdentifier());
							list.add(id);
							weights.add(list, 100);

							// add to the workspace
							ProvidedObjectHelper.publish(context, DATA_OBJ_PREFIX + " cg", cg, CoverabilityGraph.class, false);
							ProvidedObjectHelper.publish(context, DATA_OBJ_PREFIX + " we", weights, DirectedGraphElementWeights.class,
									false);
							ProvidedObjectHelper.publish(context, DATA_OBJ_PREFIX + " ss", ss, StartStateSet.class, false);
							ProvidedObjectHelper.publish(context, DATA_OBJ_PREFIX + " as", as, AcceptStateSet.class, false);
							TransitionSystemConnection c = context.getConnectionManager().addConnection(
									new TransitionSystemConnection(cg, weights, ss, as));
							c.setLabel(DATA_OBJ_PREFIX + " conn");

							try {
								// visualize the extended graph
								JComponent visualizer = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, cg);
								coverabilityGraphvisualizer.removeAll();
								coverabilityGraphvisualizer.add(visualizer, BorderLayout.CENTER);
							} catch (ConnectionCannotBeObtained e) {
								// Uhm...
							}
						}
					}
				});
			}
		});

		JPanel errorsArea = SlickerFactory.instance().createRoundedPanel(15, UIColors.lightLightGray);
		errorsArea.setLayout(new BorderLayout(0, 10));
		errorsArea.add(scrollPane, BorderLayout.CENTER);

		// coverability graph area
		coverabilityGraphvisualizer = SlickerFactory.instance().createRoundedPanel(15, UIColors.lightLightGray);
		coverabilityGraphvisualizer.setLayout(new BorderLayout(0, 0));
		coverabilityGraphvisualizer.add(nothingToDisplay, BorderLayout.CENTER);

		// main area
		JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, errorsArea, coverabilityGraphvisualizer);
		split.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		split.setOpaque(false);
		split.setDividerSize(10);
		split.setDividerLocation(400);

		split.setUI(new BasicSplitPaneUI() {
			@Override
			public BasicSplitPaneDivider createDefaultDivider() {
				return new BasicSplitPaneDivider(this) {
					private static final long serialVersionUID = 9030573699042574015L;

					@Override
					public void paint(Graphics g) {
						g.setColor(new Color(0, 0, 0, 0));
						g.fillRect(0, 0, getSize().width, getSize().height);
						super.paint(g);
					}

					@Override
					public void setBorder(Border b) {
					}
				};
			}
		});

		// add everything
		add(split, BorderLayout.CENTER);

		updateErrorList();
	}

	public void updateErrorList() {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				list.clearSelection();
				dlm.clear();
				for (Pair<Integer, Triple<State, String, State>> error : tracker.getTopErrors().getList()) {
					dlm.addElement(error);
				}
			}
		});
	}

	private CoverabilityGraph cloneModel(CoverabilityGraph originalModel) {
		CoverabilityGraph cg = new CoverabilityGraph(originalModel.getLabel());
		// clone the states
		for (State s : originalModel.getNodes()) {
			cg.addState(s.getIdentifier());
			cg.getNode(s.getIdentifier()).setAccepting(s.isAccepting());
		}
		// clone the transitions
		for (Transition transition : originalModel.getEdges()) {
			cg.addTransition(transition.getSource().getIdentifier(), transition.getTarget().getIdentifier(),
					transition.getIdentifier());
		}
		return cg;
	}
}
