package org.processmining.streamconformance.regions.plugin.cg;

import java.util.ArrayList;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.connections.transitionsystem.TransitionSystemConnection;
import org.processmining.models.graphbased.directed.DirectedGraphElementWeights;
import org.processmining.models.graphbased.directed.transitionsystem.AcceptStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.CoverabilityGraph;
import org.processmining.models.graphbased.directed.transitionsystem.StartStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.Transition;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;

public class DummyCoverabilityGraphCreator {

	protected static void addNoisyTransition(TransitionSystem ts, Object fromState, Object toState, Object identifier,
			DirectedGraphElementWeights weights) {
		ts.addTransition(fromState, toState, identifier);
		Transition transition = ts.findTransition(fromState, toState, identifier);
		ArrayList<Object> list = new ArrayList<Object>(3);
		list.add(transition.getSource().getIdentifier());
		list.add(transition.getTarget().getIdentifier());
		list.add(transition.getIdentifier());
		weights.add(list, 1);
	}

	protected static void addPlainTransition(TransitionSystem ts, Object fromState, Object toState, Object identifier) {
		ts.addTransition(fromState, toState, identifier);
	}

	@Plugin(
			name = "Create Coverability Graph 1",
			parameterLabels = {},
			returnLabels = { "Coverability Graph" },
			returnTypes = { CoverabilityGraph.class },
			userAccessible = false)
	@UITopiaVariant(
			author = "A. Burattin",
			affiliation = "",
			email = "")
	public static CoverabilityGraph create1(PluginContext context) {
		CoverabilityGraph cg = new CoverabilityGraph("test");

		cg.addState(0);
		cg.addState(1);
		cg.addState(2);
		cg.addState(3);
		cg.addState(4);
		cg.addState(5);

		cg.addTransition(0, 1, "A");
		cg.addTransition(1, 2, "tau split");
		cg.addTransition(2, 3, "B");
		cg.addTransition(1, 4, "C");
		cg.addTransition(3, 4, "tau split");
		cg.addTransition(4, 5, "D");

		return cg;
	}

	@Plugin(
			name = "Create Coverability Graph for Online conformance",
			parameterLabels = {},
			returnLabels = { "Coverability Graph", "Transition weights", "Start state set", "Accept state set" },
			returnTypes = { CoverabilityGraph.class, DirectedGraphElementWeights.class, StartStateSet.class, AcceptStateSet.class },
			userAccessible = false)
	@UITopiaVariant(
			author = "A. Burattin",
			affiliation = "",
			email = "")
	public static Object create2(PluginContext context) {
		CoverabilityGraph cg = new CoverabilityGraph("test");

		// nodes
		cg.addState(1);
		cg.addState(2);
		cg.addState(3);
		cg.addState(4);
		cg.addState(5);
		cg.addState(6);

		// start state
		StartStateSet sss = new StartStateSet();
		sss.add(1);

		// accepting state
		AcceptStateSet as = new AcceptStateSet();
		as.add(6);

		// transitions
		DirectedGraphElementWeights weights = new DirectedGraphElementWeights();

		addPlainTransition(cg, 1, 2, "A");
		addNoisyTransition(cg, 1, 3, "B", weights);
		addNoisyTransition(cg, 1, 4, "C", weights);
		addNoisyTransition(cg, 1, 6, "D", weights);

		addNoisyTransition(cg, 2, 2, "A", weights);
		addPlainTransition(cg, 2, 3, "B");
		addPlainTransition(cg, 2, 4, "C");
		addNoisyTransition(cg, 2, 6, "D", weights);

		addNoisyTransition(cg, 3, 2, "A", weights);
		addNoisyTransition(cg, 3, 3, "B", weights);
		addPlainTransition(cg, 3, 5, "C");
		addNoisyTransition(cg, 3, 6, "D", weights);

		addNoisyTransition(cg, 4, 2, "A", weights);
		addPlainTransition(cg, 4, 5, "B");
		addNoisyTransition(cg, 4, 4, "C", weights);
		addNoisyTransition(cg, 4, 6, "D", weights);

		addNoisyTransition(cg, 5, 2, "A", weights);
		addNoisyTransition(cg, 5, 5, "B", weights);
		addNoisyTransition(cg, 5, 5, "C", weights);
		addPlainTransition(cg, 5, 6, "D");

		addNoisyTransition(cg, 6, 2, "A", weights);
		addNoisyTransition(cg, 6, 5, "B", weights);
		addNoisyTransition(cg, 6, 5, "C", weights);
		addNoisyTransition(cg, 6, 6, "D", weights);

		context.addConnection(new TransitionSystemConnection(cg, weights, sss, as));

		/*
		 * Return the net and the marking.
		 */
		Object[] objects = new Object[4];
		objects[0] = cg;
		objects[1] = weights;
		objects[2] = sss;
		objects[3] = as;
		return objects;
	}
}
